/**
 * License Agreement.
 *
 * Rich Faces - Natural Ajax for Java Server Faces (JSF)
 *
 * Copyright (C) 2007 Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
/*

 * Created on 25.09.2004

 *

 * Copyright 1999-2004 The Apache Software Foundation.

 * 

 * Licensed under the Apache License, Version 2.0 (the "License");

 * you may not use this file except in compliance with the License.

 * You may obtain a copy of the License at

 * 

 *      http://www.apache.org/licenses/LICENSE-2.0

 * 

 * Unless required by applicable law or agreed to in writing, software

 * distributed under the License is distributed on an "AS IS" BASIS,

 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.

 * See the License for the specific language governing permissions and

 * limitations under the License.

 */
package org.jboss.portletbridge.context;

import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Logger;

import javax.faces.context.ExternalContext;

import org.jboss.portletbridge.util.BridgeLogger;

/**
 * 
 * @author shura
 * 
 * 
 * 
 *         Base implementation of <code>ExternalContext</code>
 * 
 * 
 */
public abstract class AbstractExternalContext extends ExternalContext {


	// TODO - optimization.
	private Map<String, Object> applicationMap;

	private Map<String, String> initParameterMap;

	protected Map<String, String> requestHeaderMap = null;

	protected Map<String, String[]> requestHeaderValues;

	private Map<String, Object> requestMap;

	private Map<String, String> requestParameterMap;

	private Map<String, String[]> requestParameterValuesMap;

	private Map<String, Object> sessionMap;

	private Object request;

	private Object response;

	// private Map<String,Object> actionSettings;

	private Object context;

	/**
	 * 
	 * @param context
	 * @param request
	 * @param response
	 * @param defaultContext
	 *            -
	 * 
	 *            default implementation of <code>ExternalFacesContext</code>.
	 * 
	 */
	public AbstractExternalContext(Object context, Object request,
			Object response) {
		super();
		this.context = context;
		this.request = request;
		this.response = response;
	}

	protected abstract String getNamespace();

	public String encodeNamespace(String name) {

		return getNamespace() + name;
	}

	/*
	 * 
	 * (non-Javadoc)
	 * 
	 * 
	 * 
	 * @see javax.faces.context.ExternalContext#dispatch(java.lang.String)
	 */
	public Map<String, Object> getApplicationMap() {
		if (this.applicationMap == null) {
			this.applicationMap = new ContextAttributesMap<Object>() {
				protected Enumeration<String> getEnumeration() {
					return getContextAttributeNames();
				}

				protected Object getAttribute(String name) {
					return getContextAttribute(name);
				}

				protected void setAttribute(String name, Object value) {
					setContextAttribute(name, value);
				}

				protected void removeAttribute(String name) {
					removeContextAttribute(name);
				}
			};
		}
		return this.applicationMap;
	}

	protected abstract void removeContextAttribute(String name);

	protected abstract void setContextAttribute(String name, Object value);

	protected abstract Object getContextAttribute(String name);

	protected abstract Enumeration<String> getContextAttributeNames();

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.faces.context.ExternalContext#getAuthType()
	 */
	public Object getContext() {
		return this.context;
	}

	/**
	 * @param context
	 */
	public void setContext(Object context) {
		this.context = context;
		this.applicationMap = null;
		this.initParameterMap = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * javax.faces.context.ExternalContext#getInitParameter(java.lang.String)
	 */
	@Override
	public Map<String, String> getInitParameterMap() {
		if (this.initParameterMap == null) {
			this.initParameterMap = new ContextAttributesMap<String>() {
				protected String getAttribute(String name) {
					return getInitParameter(name);
				}

				protected void setAttribute(String name, String value) {
					throw new UnsupportedOperationException();
				}

				protected Enumeration<String> getEnumeration() {
					return getInitParametersNames();
				}
			};
		}
		return this.initParameterMap;
	}

	/**
	 * Hoock method for initialization parameters.
	 * 
	 * @return
	 */
	protected abstract Enumeration<String> getInitParametersNames();

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.faces.context.ExternalContext#getRequest()
	 */
	public Object getRequest() {
		return this.request;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.faces.context.ExternalContext#setRequest(java.lang.Object)
	 */
	public void setRequest(Object request) {
		this.requestHeaderMap = null;
		this.requestHeaderValues = null;
		this.requestMap = null;
		this.requestParameterMap = null;
		this.requestParameterValuesMap = null;
		this.sessionMap = null;
		this.request = request;
	}

	public Map<String, Object> getRequestCookieMap() {
		// Portlet environment don't have methods to use cookies.
		return Collections.emptyMap();
	}

	public Map<String, Object> getRequestMap() {
		if (this.requestMap == null) {
			this.requestMap = new ContextAttributesMap<Object>() {
				protected Enumeration<String> getEnumeration() {
					return getRequestAttributeNames();
				}

				protected Object getAttribute(String name) {
					return getRequestAttribute(name);
				}

				protected void setAttribute(String name, Object value) {
					setRequestAttribute(name, value);
				}

				protected void removeAttribute(String name) {
					removeRequestAttribute(name);
				}
			};
		}
		return this.requestMap;
	}

	protected abstract void removeRequestAttribute(String name);

	protected abstract void setRequestAttribute(String name, Object value);

	protected abstract Object getRequestAttribute(String name);

	protected abstract Enumeration<String> getRequestAttributeNames();

	public Map<String, String> getRequestParameterMap() {
		//
		if (this.requestParameterMap == null) {
			Map<String, String> requestParameterMap = new HashMap<String, String>();
			Map<String, String[]> parameterValuesMap = getRequestParameterValuesMap();
			for(Map.Entry<String,String[]> entry: parameterValuesMap.entrySet()){
				String[] value = entry.getValue();
				if(null != value && value.length>0){
					requestParameterMap.put(entry.getKey(), value[0]);
				} else {
					requestParameterMap.put(entry.getKey(), null);
				}
			}
			this.requestParameterMap = Collections.unmodifiableMap(requestParameterMap);
		}
		return this.requestParameterMap;
	}


	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.faces.context.ExternalContext#getRequestParameterNames()
	 */
	public Iterator<String> getRequestParameterNames() {
		return getRequestParameterValuesMap().keySet().iterator();
	}

	/*
	 * 
	 * (non-Javadoc)
	 * 
	 * 
	 * 
	 * @see javax.faces.context.ExternalContext#getRequestParameterValuesMap()
	 */
	public Map<String, String[]> getRequestParameterValuesMap() {
		if (this.requestParameterValuesMap == null) {
			Map<String, String[]> requestParameterValuesMap = new HashMap<String, String[]>(getNativeRequestParameterValuesMap());
			requestParameterValuesMap.putAll(getExtraRequestParameterValuesMap());
			this.requestParameterValuesMap = Collections.unmodifiableMap(requestParameterValuesMap);
		}
		return this.requestParameterValuesMap;
	}

	protected abstract Map<String, String[]> getNativeRequestParameterValuesMap();

	protected abstract Map<String, String[]> getExtraRequestParameterValuesMap();


	/*
	 * 
	 * (non-Javadoc)
	 * 
	 * 
	 * 
	 * @see javax.faces.context.ExternalContext#getResponse()
	 */
	public void setResponse(Object response) {
		this.response = response;
	}

	public Object getResponse() {
		return this.response;
	}

	/*
	 * 
	 * (non-Javadoc)
	 * 
	 * 
	 * 
	 * @see javax.faces.context.ExternalContext#getSessionMap()
	 */
	public Map<String, Object> getSessionMap() {
		if (this.sessionMap == null) {
			this.sessionMap = new ContextAttributesMap<Object>() {

				protected Enumeration<String> getEnumeration() {
					return getSessionAttributeNames();
				}

				protected Object getAttribute(String name) {
					return getSessionAttribute(name);
				}

				protected void setAttribute(String name, Object value) {
					setSessionAttribute(name, value);
				}

				protected void removeAttribute(String name) {
					removeSessionAttribute(name);
				}

			};
		}
		return this.sessionMap;
	}

	protected abstract void removeSessionAttribute(String name);

	protected abstract void setSessionAttribute(String name, Object value);

	protected abstract Object getSessionAttribute(String name);

	protected abstract Enumeration<String> getSessionAttributeNames();
}
