/******************************************************************************
 * JBoss, a division of Red Hat                                               *
 * Copyright 2006, Red Hat Middleware, LLC, and individual                    *
 * contributors as indicated by the @authors tag. See the                     *
 * copyright.txt in the distribution for a full listing of                    *
 * individual contributors.                                                   *
 *                                                                            *
 * This is free software; you can redistribute it and/or modify it            *
 * under the terms of the GNU Lesser General Public License as                *
 * published by the Free Software Foundation; either version 2.1 of           *
 * the License, or (at your option) any later version.                        *
 *                                                                            *
 * This software is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU           *
 * Lesser General Public License for more details.                            *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public           *
 * License along with this software; if not, write to the Free                *
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA         *
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.                   *
 ******************************************************************************/
package org.jboss.portletbridge.gatein;

import java.util.Map;

import javax.faces.context.FacesContext;
import javax.portlet.RenderRequest;

import org.gatein.pc.portlet.impl.jsr168.api.RenderRequestImpl;
import org.jboss.portletbridge.BridgeConfig;
import org.jboss.portletbridge.BridgeStrategy;
import org.jboss.portletbridge.BridgeStrategyWrapper;

/**
 * @author <a href="mailto:whales@redhat.com">Wesley Hales</a>
 */
public class GateinStrategy extends BridgeStrategyWrapper {
	private final BridgeStrategy parent;

	public GateinStrategy(BridgeConfig config, BridgeStrategy parent) {
		super(config);
		// Assert GateIn internal class.
		RenderRequestImpl.class.getName();
		this.parent = parent;
	}

	@Override
	public void beforeRenderRequest(FacesContext facesContext) {

		Object responseObject = facesContext.getExternalContext().getRequest();
		if (responseObject instanceof RenderRequest) {
			RenderRequestImpl renderRequest = (RenderRequestImpl) responseObject;
			Map<String, String[]> servletParameterMap = renderRequest
					.getRealRequest().getRequest().getParameterMap();
			for (String name : servletParameterMap.keySet()) {
				String[] values = servletParameterMap.get(name);
				facesContext.getExternalContext().getRequestMap()
						.put(name, values);
			}
		}
		super.beforeRenderRequest(facesContext);
	}

	@Override
	protected BridgeStrategy getWrapped() {
		return parent;
	}

}
