package org.jboss.portletbridge.richfaces;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.faces.FacesException;
import javax.faces.context.FacesContext;

import org.ajax4jsf.Messages;
import org.ajax4jsf.resource.GifRenderer;
import org.ajax4jsf.resource.HTMLRenderer;
import org.ajax4jsf.resource.InternetResource;
import org.ajax4jsf.resource.InternetResourceBase;
import org.ajax4jsf.resource.JpegRenderer;
import org.ajax4jsf.resource.LogfileRenderer;
import org.ajax4jsf.resource.MimeRenderer;
import org.ajax4jsf.resource.PngRenderer;
import org.ajax4jsf.resource.ResourceBuilderImpl;
import org.ajax4jsf.resource.ResourceNotFoundException;
import org.ajax4jsf.resource.ResourceRenderer;
import org.ajax4jsf.resource.StaticResource;
import org.ajax4jsf.resource.TemplateCSSRenderer;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Extend RichFaces ResourceBuilderImpl for customization.
 * Override createStaticResource() to resolve issue of existing code relying on finding
 * a ServletContext, which is not possible within a Portlet.
 *
 * @author kenfinnigan
 */
public class RichFacesResourceBuilder extends ResourceBuilderImpl {
	private static final Log log = LogFactory.getLog(RichFacesResourceBuilder.class);

	private static final ResourceRenderer defaultRenderer = new MimeRenderer(null);

	private Map<String, ResourceRenderer> renderers;

	public RichFacesResourceBuilder() {
		super();
		renderers = new HashMap<String, ResourceRenderer>();
		// append known renderers for extentions.
		renderers.put(".gif", new GifRenderer());
		ResourceRenderer renderer = new JpegRenderer();
		renderers.put(".jpeg", renderer);
		renderers.put(".jpg", renderer);
		renderers.put(".png", new PngRenderer());
		renderers.put(".js", getScriptRenderer());
		renderers.put(".css", getStyleRenderer());
		renderers.put(".log", new LogfileRenderer());
		renderers.put(".html", new HTMLRenderer());
		renderers.put(".xhtml", new MimeRenderer("application/xhtml+xml"));
		renderers.put(".xml", new MimeRenderer("text/xml"));
		renderers.put(".xcss", new TemplateCSSRenderer());
		renderers.put(".swf", new MimeRenderer("application/x-shockwave-flash"));
	}

	/**
	 * Build resource for link to static context in webapp.
	 * 
	 * @param path
	 * @return
	 * @throws FacesException
	 */
	@Override
	protected InternetResource createStaticResource(String path)
			throws ResourceNotFoundException, FacesException {
		FacesContext context = FacesContext.getCurrentInstance();
		if (null != context) {
			try {
				URL in = context.getExternalContext().getResource(path);
				if (null != in) {
					InternetResourceBase res = new StaticResource(path);
					setRenderer(res, path);
					// Detect last modification time.
					res.setLastModified(new Date(getStartTime()));
					addResource(path, res);
					return res;
				}
			} catch (MalformedURLException e) {
				throw new ResourceNotFoundException(Messages.getMessage(
						Messages.STATIC_RESOURCE_NOT_FOUND_ERROR, path),e);
			}
		}
		throw new ResourceNotFoundException(Messages.getMessage(
				Messages.STATIC_RESOURCE_NOT_FOUND_ERROR, path));
	}

	private void setRenderer(InternetResource res, String path)
			throws FacesException {
		int lastPoint = path.lastIndexOf('.');
		if (lastPoint > 0) {
			String ext = path.substring(lastPoint);
			ResourceRenderer resourceRenderer =  getRendererByExtension(ext);
			if (null != resourceRenderer) {
				res.setRenderer(resourceRenderer);
			} else {
				if (log.isDebugEnabled()) {
					log.debug(Messages.getMessage(
							Messages.NO_RESOURCE_REGISTERED_ERROR_2, path,
							renderers.keySet()));
				}

				// String mimeType = servletContext.getMimeType(path);
				res.setRenderer(defaultRenderer);
			}
		}
	}

}
