package org.jboss.portletbridge.context;

import java.util.Enumeration;
import java.util.Locale;

import javax.portlet.PortletContext;
import javax.portlet.PortletRequest;

import junit.framework.TestCase;

import org.jboss.portletbridge.MockPortletRequest;

/**
 * @author kenfinnigan
 *
 */
public class PortletHeaderMapTest extends TestCase {

	class MockHeaderPortletRequest extends MockPortletRequest {
		private Locale[] locales = null;

		public MockHeaderPortletRequest(PortletContext portletContext, Locale[] locales) {
			super(portletContext);
			this.properties.put("Accept-Language", null);
			this.locales = locales;
		}

		@Override
		public Enumeration<Locale> getLocales() {
			return new Enumeration<Locale>() {
				int count = 0;

				public boolean hasMoreElements() {
					if (locales.length > count) {
						return true;
					}
					return false;
				}

				public Locale nextElement() {
					return locales[count++];
				}				
			};
		}
	}

	public void testSingleLocale() throws Exception {
		Locale[] locales = {Locale.ENGLISH};
		PortletRequest req = new MockHeaderPortletRequest(null, locales);
		PortletHeaderMap<String> headerMap = new RequestHeaderMap(req, false);
		assertEquals("en", headerMap.get("Accept-Language"));
	}

	public void testMultipleLocales() throws Exception {
		Locale[] locales = {Locale.ENGLISH, Locale.US};
		PortletRequest req = new MockHeaderPortletRequest(null, locales);
		PortletHeaderMap<String> headerMap = new RequestHeaderMap(req, false);
		assertEquals("en,en-US", headerMap.get("Accept-Language"));
	}
}
