/**
 * 
 */
package org.jboss.portletbridge.example.seam;

import java.io.Serializable;
import java.util.Date;
import java.util.EventListener;
import java.util.EventObject;
import java.util.UUID;

import org.ajax4jsf.event.PushEventListener;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;

/**
 * @author Ilya Shaikovsky
 * 
 */
@Name("push")
@Scope(ScopeType.SESSION)
public class PushBean implements Runnable,Serializable{

    private String uuid = "";
    
    private boolean enabled = false;
    
    private Date startDate; 
    
    PushEventListener listener;

    private volatile Thread thread;

    // private int eventsFired counter;

    public void addListener(EventListener listener) {
        synchronized (listener) {
            if (this.listener != listener) {
                this.listener = (PushEventListener) listener;
            }
        }
    }

    public void run() {
        while (thread != null) {
            try {
                if (((new Date()).getTime()-startDate.getTime())>=60000) {
                    stop();
                }
                uuid = UUID.randomUUID().toString();
                listener.onEvent(new EventObject(this));
                Thread.sleep(10000);
            } catch (InterruptedException e) {
                // TODO Auto-generated catch block
                // e.printStackTrace();
            }
        }
    }

    public String getUuid() {
        return uuid;
    }

    public void start() {
        if (thread == null) {
            setStartDate(new Date());
            thread = new Thread(this);
            thread.setDaemon(true);
            thread.start();
            setEnabled(true);
        }
    }

    public void stop() {
        if (thread != null) {
            //thread.stop();
        	// setStartDate(null);
            setEnabled(false);
            thread = null;
        }
    }

    public Thread getThread() {
        return thread;
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }
}